import "Turbine.UI.Lotro";

-- Quickslot class override. Quickslot is blanked if its shortcut is dragged out.
-- TODO: swap shortcuts
QuickSlot = class( Turbine.UI.Lotro.Quickslot );

function QuickSlot:Constructor()
	Turbine.UI.Lotro.Quickslot.Constructor( self );

	self.pressed = false;
	self.dragging = false;
	
	-- event
	self.Clicked = Equendil.Utils.Event;
	
	-- Drag & drop is possibly initiated by pressing the left mouse button ...
	self.MouseDown = function( sender, args )
		if args.Button == Turbine.UI.MouseButton.Left then
			self.pressed = true;
		end
	end
	-- Drag & drop terminates when the left mouse button is released ...
	-- however it could just be a click, so the quickslot is only cleared if the pointer left the quickslot since the left mouse button was pressed
	self.MouseUp = function( sender, args )
		-- we get a mouse up event when drag & dropping with the mouse button still down .... can't rely on that event
		-- still clearing the "pressed" button status and the shortcut to keep this working ...
		if args.Button == Turbine.UI.MouseButton.Left then
			if self.dragging then -- did the mouse leave the quickslot ?
				self:SetShortcut(nil); -- blank the shortcut, this will trigger a ShortCutChanged() event
				self:SetVisible(true); -- hmm seems to go invisble after removing the shortcut
			end
		end -- resets flags
		self.pressed = false;
		self.dragging = false;
	end
	
	-- if the pointer leaves the quickslot since the left mouse button was pressed, we'll assume it's a proper drag & drop action
	self.MouseLeave = function( sender, args )
		if self.pressed then
			self.dragging = true;
		end
	end

	-- Relay mouse click events
	self.MouseClick = function( sender, args )
		if self.Clicked ~= nil then
			self.Clicked( args.Button );
		end
	end
	
--	self.ShortcutChanged = function( sender, args )
--		Turbine.Log:Information( "Quickslot: changed" );
--		local shortcut = sender:GetShortcut();
--		local data = shortcut:GetData();
--		Turbine.Log:Information( data );
--		local t = shortcut:GetType();

--		if t == Turbine.UI.Lotro.ShortcutType.Item then
			--Turbine.Log:Information( "item" );
		--end
--		local item = shortcut:GetItem();
--		shortcut:GetIconImageID();
--		if item ~= nil then Turbine.Log:Information( item:GetIconId() ); end
--	end

end

-- overloading SetShortcut so it doesn't fail if we set a shortcut that's not valid (item sold to a vendor for instance)
function QuickSlot:SetShortcut( shortcut )
	local success, rv = pcall( Turbine.UI.Lotro.Quickslot.SetShortcut, self, shortcut );
	return rv;
end